package org.example;

import java.io.*;
import java.net.*;
import java.util.*;

import javax.xml.bind.JAXBContext;

import com.sun.ejte.ccl.reporter.SimpleReporterAdapter;

public class WebTest {

    private static SimpleReporterAdapter stat
        = new SimpleReporterAdapter("appserver-sqe");
    private static String TEST_NAME = "test";

    private String testSuiteID;
    private String host;
    private String port;
    private String contextRoot;
    private String urlPath;
    private JAXBContext jc = null;

    public WebTest(String[] args) {
        testSuiteID = args[0];
        host = args[1];
        port = args[2];
        contextRoot = args[3];
        urlPath = "http://" + host + ":" + port + contextRoot + 
                     "/rest/customers";
        TEST_NAME = testSuiteID;
    }
    
    public static void main(String[] args) {
        stat.addDescription("jaxbri tests with jaxrs and jpa");
        WebTest webTest = new WebTest(args);
        webTest.doTest();
        stat.printSummary(TEST_NAME);
    }

    public void doTest() {     
        try {

          // Setup test data 
          jpatest("initData");

          // jpa query for dbg
          // jpatest("queryById");
          // jpatest("queryByCity");

          // start Moxy tests with using JAXBContext
	  jc = JAXBContext.newInstance(Customer.class); 
          testGet();           // READ operation: GET
          testPost();          // INSERT operation:  POST   
          testPut();           // UPDATE operation: PUT
          testDelete();        // DELETE operation: DELETE 
        } catch (Exception ex) {
	    stat.addStatus(TEST_NAME, stat.FAIL);
            ex.printStackTrace();
        }
    }

    private void jpatest(String c) throws Exception {
        String EXPECTED_RESPONSE = c +":pass";
        boolean result=false;
        String url = "http://" + host + ":" + port 
                     + contextRoot + "/data?tc="+c;
        System.out.println("\n***url="+url);

        HttpURLConnection conn = (HttpURLConnection)
	                  (new URL(url)).openConnection();

        int code = conn.getResponseCode();

        if (code != 200) {
            System.err.println("Unexpected return code: " + code);
	} else {
            InputStream is = conn.getInputStream();
            BufferedReader input = new 
	      BufferedReader(new InputStreamReader(is));
	    String line = null;
	    while ((line = input.readLine()) != null) {
	      if (line.contains(EXPECTED_RESPONSE)) {
                result = true;
		break;
	      }
	    }
        }    

	if (result) {
	  stat.addStatus(TEST_NAME + c, stat.PASS);
	} else {
	  stat.addStatus(TEST_NAME + c, stat.FAIL);
	}
	
    }

    private void testGet() throws Exception {
        String TEST_CASE = TEST_NAME + "GET/READ";
        boolean pass = false;
        Customer customer = null;

        // Get obj with id=1 as xml
        String expectedLastName = "Doe";  // 
	String urlString = urlPath + "/1";  // id = 1 

        System.out.println("\n***urlString="+urlString);
	HttpURLConnection conn = null;

        try {
	  conn = (HttpURLConnection)(new URL(urlString)).openConnection();

	  // Test GET
	  conn.setRequestMethod("GET");
	  conn.setRequestProperty("Accept", "application/xml");
	  InputStream xml = conn.getInputStream();

	  customer = (Customer) 
	  getJAXBContext().createUnmarshaller().unmarshal(xml);

          System.out.println("xml from Get="+xml);
	  System.out.println("obj from unmarshal xml ="+customer);

	  if (customer != null) {
	    String c_lastName = customer.getLastName();
	    if (c_lastName.equals(expectedLastName)){
	      pass = true;
	    }
	    System.out.println("expected: " + expectedLastName + 
			       ", got: " + c_lastName);
	  }
          checkXML(urlString);
        } catch (Exception ex) {
	  System.out.println("Got Ex");
          ex.printStackTrace();
	} finally {
	  conn.disconnect();
	} 

	if (pass){
	    stat.addStatus(TEST_CASE, stat.PASS);
	} else {
	    stat.addStatus(TEST_CASE, stat.FAIL);
	}

    }

    private void testPost() throws Exception {
        String TEST_CASE = TEST_NAME + "POST/INSERT";
        boolean pass = false;
        String urlString = urlPath;
        System.out.println("\n***urlString="+urlString);

        HttpURLConnection conn = null;

        try {
	  conn = (HttpURLConnection)(new URL(urlString)).openConnection();
	  conn.setDoOutput(true);
	  conn.setInstanceFollowRedirects(false);

	  // Test POST
	  conn.setRequestMethod("POST");

	  conn.setRequestProperty("Content-Type", "application/xml");
	  OutputStream os = conn.getOutputStream();

          // put obj with id=4
	  getJAXBContext().createMarshaller().marshal(getControlObject4(), os);

	  os.flush();  
	  conn.getResponseCode();
	  Customer testObject = verifyPostPut(4);
	  String controlFirstName = getControlObject4().getFirstName();
          String testFirstName = testObject.getFirstName();
	  if (controlFirstName.equals(testFirstName)){
	      pass = true;
	  }
	  System.out.println("expected: " + controlFirstName + 
			     ", got: " + testFirstName);

          checkXML(urlString + "/4");
	} catch (Exception ex) {
	  System.out.println("Got Ex");
	} finally {
	  conn.disconnect();
	} 

	if (pass){
	    stat.addStatus(TEST_CASE, stat.PASS);
	} else {
	    stat.addStatus(TEST_CASE, stat.FAIL);
	}

    }


    private void testPut() throws Exception {
        String TEST_CASE = TEST_NAME + "PUT/UPDATE";
        boolean pass = false;
        String urlString = urlPath;
        System.out.println("\n***urlString="+urlString);

        HttpURLConnection conn = null;

        try {
	  conn = (HttpURLConnection)(new URL(urlString)).openConnection();
	  conn.setDoOutput(true);
	  conn.setInstanceFollowRedirects(false);

	  // Test PUT
	  conn.setRequestMethod("PUT");

	  conn.setRequestProperty("Content-Type", "application/xml");
	  OutputStream os = conn.getOutputStream();

          // put obj with id=4
	  getJAXBContext().createMarshaller().marshal(getUpdatedControlObject4(), os);

	  os.flush();  
          int responseCode = conn.getResponseCode();
	  System.out.println("expected: 204" + 
			     ", got: " + responseCode);
	  Customer testObject = verifyPostPut(4);
	  String controlFirstName = getUpdatedControlObject4().getFirstName();
	  String controlCityName = getUpdatedControlObject4().getAddress().getCity();
          String testFirstName = testObject.getFirstName();
          String testCityName = testObject.getAddress().getCity();
	  if (controlFirstName.equals(testFirstName) &&
              controlCityName.equals(testCityName)){
	      pass = true;
	  }
	  System.out.println("expected: " + controlFirstName + 
			     ", got: " + testFirstName);
	  System.out.println("expected: " + controlCityName + 
			     ", got: " + testCityName);
          checkXML(urlString + "/4");
	} catch (Exception ex) {
	  System.out.println("Got Ex");
	} finally {
	  conn.disconnect();
	} 

	if (pass){
	    stat.addStatus(TEST_CASE, stat.PASS);
	} else {
	    stat.addStatus(TEST_CASE, stat.FAIL);
	}

    }

    private void testDelete() throws Exception {
        String TEST_CASE = TEST_NAME + "DELETE";
        boolean pass = false;
        String urlString = urlPath + "/" + 4;
        System.out.println("\n***urlString="+urlString);

        HttpURLConnection conn = null;

        try {
	  conn = (HttpURLConnection)(new URL(urlString)).openConnection();

	  // Test DELETE
	  conn.setRequestMethod("DELETE");
          int responseCode = conn.getResponseCode();

          if (responseCode == 204 && !checkXML(urlString)) {
	      pass = true;
	  }
	  System.out.println("expected: 204" + 
			     ", got: " + responseCode);
	} catch (Exception ex) {
	  System.out.println("Got Ex");
	} finally {
	  conn.disconnect();
	} 

	if (pass){
	    stat.addStatus(TEST_CASE, stat.PASS);
	} else {
	    stat.addStatus(TEST_CASE, stat.FAIL);
	}

    }


    protected Customer getControlObject4() {
        Customer customer = new Customer();
        customer.setId(4);
        customer.setFirstName("Sarah");
        customer.setLastName("Smith");
        

        return customer;
    }  

    protected Customer getUpdatedControlObject4() {
        Customer customer = new Customer();
        customer.setId(4);
        customer.setFirstName("Sarah");
        customer.setLastName("Smith");
        
        Address address = new Address();
        address.setId(3);
        address.setStreet("1 Nowhere Drive");
        address.setCity("Ottawa");
        customer.setAddress(address);
        return customer;
    }  

    public Customer verifyPostPut(int id) 
      throws Exception {
        String urlString = urlPath + "/" + id;
        System.out.println("\n***urlString="+urlString);      
        URL url = new URL(urlString);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();
        conn.setRequestMethod("GET");
        conn.setRequestProperty("Accept", "application/xml");

        InputStream xml = conn.getInputStream();
 
        Customer testObject = 
	  (Customer) getJAXBContext().createUnmarshaller().unmarshal(xml);

        System.out.println("xml from Get="+xml);

        conn.disconnect();
        return testObject;

    }          

    public boolean checkXML(String urlString) throws Exception {
        boolean found = false;
        System.out.println("checkXML: urlString="+urlString);      
        URL url = new URL(urlString);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();
        conn.setRequestMethod("GET");
        conn.setRequestProperty("Accept", "application/xml");
        InputStream xml = conn.getInputStream(); 
        
	BufferedReader input = new BufferedReader(new InputStreamReader(xml));
        String line = null;
	while ((line = input.readLine()) != null) {
	  System.out.println("line="+line);
          found = true;
	}
        xml.close();
        conn.disconnect();
        if (found) {
	  System.out.println("checkXML: got xml");
	} else {
	  System.out.println("checkXML: not got xml");
	}
        return found;

    } 

    protected JAXBContext getJAXBContext() {
        return jc;
    }

    protected String getURLPath() {
        return urlPath;
    }

}

